import sys
from random import randint, shuffle

if (len(sys.argv) != 3):
  print('Usage: generate.py size difficulty')
  print('size: [2|3]')
  print('difficulty: [easy|medium|hard]')
  exit()

size, difficulty = sys.argv[1], sys.argv[2]

size = int(size)

if not size in [2, 3]:
  print('wrong size given')
  exit()

if not difficulty in ['easy', 'medium', 'hard']:
  print('wrong difficulty given')
  exit()

############################################################################

# medium -> 5 (default) ; easy -> 1 ; hard -> 5
difficultyLevel = 5;
if difficulty == 'easy':
   difficultyLevel = 1
if difficulty == 'hard':
   difficultyLevel = 10

# draw grid (array style)
def drawGrid(grid):
  for row in range(len(grid)):
    for col in range(len(grid[row])):
      if grid[row][col] != 0:
        sys.stdout.write(str(grid[row][col]))
      else:
        sys.stdout.write(' ')
    sys.stdout.write('\n')
  sys.stdout.write('\n')

# draw grid (inline style)
def drawGridInline(grid):
  for row in range(len(grid)):
    for col in range(len(grid[row])):
      sys.stdout.write(str(grid[row][col]))
  sys.stdout.write('\n')

#initialise empty grid
def generateEmptyGrid(size):
  emptyGrid = []
  for i in range(size * size):
    emptyGrid.append([])
    for j in range(size * size):
      emptyGrid[i].append(0)
  return emptyGrid

#A check if the grid is full
def checkFullyCompletedGrid(grid):
  for row in range(len(grid)):
    for col in range(len(grid[row])):
      if grid[row][col] == 0:
        return False
  return True

# (deep) copy of grid
def copyGrid(grid):
  copiedGrid = []
  for row in range(len(grid)):
     copiedGrid.append([])
     for col in range(len(grid[row])):
        copiedGrid[row].append(grid[row][col])
  return copiedGrid

#A backtracking/recursive function to check all possible combinations of numbers until a solution is found
def solveGrid(grid):
  gridSize = len(grid)
  cellsCount = len(grid) * len(grid[0])
  numberList = [(value + 1) for value in range(gridSize)]

  global solutionsCount

  #Find next empty cell
  for i in range(0, cellsCount):
    row = i // gridSize
    col = i % gridSize
    if grid[row][col] == 0:
      shuffle(numberList)
      for value in numberList:
        #Check that this value has not already be used on this row
        if not(value in grid[row]):
          #Check that this value has not already be used on this column
          foundInColumn = False
          for r in range(0, gridSize):
            if (value == grid[r][col]):
              foundInColumn = True

          if not foundInColumn:
            # Get sub-square
            blockColFrom = size * int(col / size)
            blockRowFrom = size * int(row / size)
            square = [grid[i][blockColFrom:blockColFrom + size] for i in range(blockRowFrom, blockRowFrom + size)]

            #Check that this value has not already be used on this sub square
            if not any(value in squareLine for squareLine in square):
              grid[row][col] = value
              if checkFullyCompletedGrid(grid):
                solutionsCount += 1
                break
              else:
                if solveGrid(grid):
                  return True
      break
  grid[row][col] = 0

#A backtracking/recursive function to check all possible combinations of numbers until a solution is found
def fillGrid(grid):
  gridSize = len(grid)
  cellsCount = len(grid) * len(grid[0])
  numberList = [(value + 1) for value in range(gridSize)]

  global solutionsCount

  #Find next empty cell
  for i in range(0, cellsCount):
    row = i // gridSize
    col = i % gridSize
    if grid[row][col] == 0:
      shuffle(numberList)
      for value in numberList:
        #Check that this value has not already be used on this row
        if not(value in grid[row]):
          #Check that this value has not already be used on this column
          foundInColumn = False
          for r in range(0, gridSize):
            if (value == grid[r][col]):
              foundInColumn = True

          if not foundInColumn:
            # Get sub-square
            blockColFrom = size * int(col / size)
            blockRowFrom = size * int(row / size)
            square = [grid[i][blockColFrom:blockColFrom + size] for i in range(blockRowFrom, blockRowFrom + size)]

            #Check that this value has not already be used on this sub square
            if not any(value in squareLine for squareLine in square):
              grid[row][col] = value
              if checkFullyCompletedGrid(grid):
                return True
              else:
                if fillGrid(grid):
                  return True
      break
  grid[row][col] = 0

solutionsCount = 1
def computeResolvableGrid(grid, wantedAttempts):
  gridSize = size * size
  global solutionsCount

  # A higher number of attempts will end up removing more numbers from the grid
  # Potentially resulting in more difficiult grids to solve!

  # Start Removing Numbers one by one
  attempts = wantedAttempts
  while attempts > 0:
    # Select a random cell that is not already empty
    row = randint(0, gridSize - 1)
    col = randint(0, gridSize - 1)
    while grid[row][col] == 0:
      row = randint(0, gridSize - 1)
      col = randint(0, gridSize - 1)

    # Remove value in this random cell
    savedCellValue = grid[row][col]
    grid[row][col] = 0

    solutionsCount = 0
    solveGrid(copyGrid(grid))

    # Non unique solution => restore this cell value
    if solutionsCount != 1:
      grid[row][col] = savedCellValue
      attempts -= 1

grid = generateEmptyGrid(size)

sys.stdout.write('Solved grid:\n')
fillGrid(grid)
drawGrid(grid)

computeResolvableGrid(grid, difficultyLevel)

sys.stdout.write('Generated grid:\n')
drawGrid(grid)

sys.stdout.write('Inline grid:\n')
drawGridInline(grid)
